---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca (2013).  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
 
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--USE ieee.numeric_std.ALL;
 
ENTITY tb_serial_mult IS
END tb_serial_mult;
 
ARCHITECTURE behavior OF tb_serial_mult IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT serial_mult
    PORT(
         clock : IN  std_logic;
         resetn : IN  std_logic;
         s : IN  std_logic;
         dA : IN  std_logic_vector(3 downto 0);
         dB : IN  std_logic_vector(3 downto 0);
         P : OUT  std_logic_vector(7 downto 0);
         done : OUT  std_logic
        );
    END COMPONENT;
    

   --Inputs
   signal clock : std_logic := '0';
   signal resetn : std_logic := '0';
   signal s : std_logic := '0';
   signal dA : std_logic_vector(3 downto 0) := (others => '0');
   signal dB : std_logic_vector(3 downto 0) := (others => '0');

 	--Outputs
   signal P : std_logic_vector(7 downto 0);
   signal done : std_logic;

   -- Clock period definitions
   constant clock_period : time := 10 ns;
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: serial_mult PORT MAP (
          clock => clock,
          resetn => resetn,
          s => s,
          dA => dA,
          dB => dB,
          P => P,
          done => done
        );

   -- Clock process definitions
   clock_process :process
   begin
		clock <= '0';
		wait for clock_period/2;
		clock <= '1';
		wait for clock_period/2;
   end process;
 

   -- Stimulus process
   stim_proc: process
   begin		
      -- hold reset state for 100 ns.
      wait for 100 ns; resetn <= '1';

      -- insert stimulus here 
		dA <= "1101"; dB <= "1011"; s <= '1'; wait for clock_period;
		s <= '0';
		
		wait for clock_period*10;
		dA <= "1001"; dB <= "1111"; s <= '1'; wait for clock_period;
		s <= '0';
      wait;
   end process;

END;
